/* abc - The AspectBench Compiler
 * Copyright (C) 2004 Julian Tibble
 *
 * This compiler is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This compiler is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this compiler, in the file LESSER-GPL;
 * if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package abc.ltl;

import java.util.Collection;
import java.util.List;

import abc.aspectj.parse.AbcLexer;
import abc.aspectj.parse.LexerAction_c;

/*
 * @author Eric Bodden
 *
 */
public class AbcExtension extends abc.meta.AbcExtension
{
    public static final String FORMULA_TERM_CONSTRUCTOR = "LTL";

    private abc.aspectj.ExtensionInfo extensionInfo;
    
    protected void collectVersions(StringBuffer versions)
    {
        super.collectVersions(versions);
        versions.append(" with JLO " +
                        new abc.ltl.Version().toString() +
                        "\n");
    }

    public abc.aspectj.ExtensionInfo
            makeExtensionInfo(Collection jar_classes,
                              Collection aspect_sources)
    {
    	this.extensionInfo = new abc.ltl.ExtensionInfo(jar_classes, aspect_sources);
        return this.extensionInfo;
    }
    
    public abc.aspectj.ExtensionInfo
    		getExtensionInfo() {
    	return this.extensionInfo;
    }

    
    protected List/*<ShadowType>*/ listShadowTypes()
    {
        List/*<ShadowType*/ shadowTypes = super.listShadowTypes();

//        shadowTypes.add(SyncShadowMatch.shadowType());
//        shadowTypes.add(ThrowShadowMatch.shadowType());
        //TODO add additional Shadow Types
        
        return shadowTypes;
    }

    public void addBasicClassesToSoot()
    {
        super.addBasicClassesToSoot();
//
//        Scene.v().addBasicClass("org.aspectbench.eaj.runtime.reflect.EajFactory",
//                                SootClass.SIGNATURES);
        //TODO needed at all?
    }

    public String runtimeSJPFactoryClass() {
        return "org.aspectbench.eaj.runtime.reflect.EajFactory";
    }

    /* (non-Javadoc)
     * @see abc.main.AbcExtension#initLexerKeywords(abc.aspectj.parse.AbcLexer)
     */
    public void initLexerKeywords(AbcLexer lexer) {
                // Add the base keywords
                super.initLexerKeywords(lexer);

        lexer.addAspectJKeyword(FORMULA_TERM_CONSTRUCTOR, new LexerAction_c(new Integer(abc.ltl.parse.sym.LTL_DECL_PREFIX),
        		new Integer(lexer.pointcut_state())));

        lexer.addPointcutKeyword("entry", new LexerAction_c(new Integer(abc.ltl.parse.sym.LTL_ENTRY)));

        lexer.addPointcutKeyword("exit", new LexerAction_c(new Integer(abc.ltl.parse.sym.LTL_EXIT)));

        lexer.addPointcutKeyword("X", new LexerAction_c(new Integer(abc.ltl.parse.sym.LTL_NEXT)));
        
        lexer.addPointcutKeyword("F", new LexerAction_c(new Integer(abc.ltl.parse.sym.LTL_FINALLY)));

        lexer.addPointcutKeyword("G", new LexerAction_c(new Integer(abc.ltl.parse.sym.LTL_GLOBALLY)));

        lexer.addPointcutKeyword("U", new LexerAction_c(new Integer(abc.ltl.parse.sym.LTL_UNTIL)));

        lexer.addPointcutKeyword("R", new LexerAction_c(new Integer(abc.ltl.parse.sym.LTL_RELEASE)));
        
        //IMPLIES and EQUIV are handled as operators as a patch to the "flex" file
    }
}
