package abc.ltl.ast;

import java.util.IdentityHashMap;
import java.util.Map;

import polyglot.ast.AmbTypeNode;
import polyglot.ast.TypeNode;
import abc.aspectj.ast.Pointcut;
import abc.ltl.Util;

public class EntryExit {

    private static Map<Pointcut,EntryExitKind> isEntryExit = new IdentityHashMap<Pointcut, EntryExitKind>();
    
    public static void entry(Pointcut pc) {
        isEntryExit.put(pc,new Entry());
    }
    
    public static void exit(Pointcut pc) {
        isEntryExit.put(pc,new Exit(Exit.ExitKind.NONE, null));
    }
        
    public static void exit(Pointcut pc, Exit.ExitKind kind, TypeNode type) {
        isEntryExit.put(pc,new Exit(kind, type));
    }

    public static EntryExitKind getEntryExit(Pointcut pc) {
        if(!isEntryExit.containsKey(pc)) {
            throw new RuntimeException("No entry/exit spec found for pointcut " +pc);
        }
        return isEntryExit.get(pc);
    }
    
    public static boolean isEntry(Pointcut pc) {
        if(!isEntryExit.containsKey(pc)) {
            throw new RuntimeException("No entry/exit spec found for pointcut " +pc);
        }
        return isEntryExit.get(pc) instanceof Entry;
    }
    
    public static boolean isExit(Pointcut pc) {
        if(!isEntryExit.containsKey(pc)) {
            throw new RuntimeException("No entry/exit spec found for pointcut " +pc);
        }
        return isEntryExit.get(pc) instanceof Exit;
    }
    
    public static String pcToString(Pointcut pc) {
        if(!isEntryExit.containsKey(pc)) {
            throw new RuntimeException("No entry/exit spec found for pointcut " +pc);
        }
        String pcLabel = Util.nodeToString(pc);
        if(isEntryExit.get(pc) instanceof Entry) {
            return "entry(" + pcLabel + ")";
        } else {
            //exit
            Exit exitInfo = (Exit)isEntryExit.get(pc);
            return "exit(" + pcLabel + ")" + exitInfo.toString();
        }
    }
    
    public static interface EntryExitKind {
    }
    
    public static class Entry implements EntryExitKind {}
    
    public static class Exit implements EntryExitKind {
        
        public enum ExitKind {RETURNING, THROWING, NONE};
        
        private TypeNode type;
        
        private ExitKind kind;
        
        private Exit(ExitKind kind, TypeNode type) {
            this.type = type;
            this.kind = kind;
        }

        public String getTypeName() {
            TypeNode type = getType();
            if(!(type instanceof AmbTypeNode)) {
                throw new RuntimeException("internal error in 'PropositionAdvice'");
            }
            AmbTypeNode amb = (AmbTypeNode) type;
            return amb.name();
        }
        
        public TypeNode getType() {
            //if neither returning nor throwing there is no type
            assert !this.kind.equals(ExitKind.NONE);
            return type;
        }

        public ExitKind getKind() {
            return kind;
        }
        
        /**
         * {@inheritDoc}
         */
        @Override
        public String toString() {
            StringBuffer result = new StringBuffer();
            if(kind.equals(ExitKind.RETURNING)) {
                result.append("returning ");
                result.append(getTypeName());
            } else if(kind.equals(ExitKind.THROWING)) {
                result.append("throwing ");
                result.append(getTypeName());
            }
            return result.toString();
        }
        
    }
}
