package abc.ltl.ast;

import java.util.Collection;
import java.util.List;

import polyglot.ast.Formal;
import polyglot.util.Position;
import rwth.i2.ltlrv.management.ByteCodePosition;
import abc.aspectj.ast.AJDelFactory;
import abc.aspectj.ast.AJExtFactory;
import abc.aspectj.ast.ArgPattern;
import abc.aspectj.ast.PCArgs;
import abc.aspectj.ast.PCTarget;
import abc.aspectj.ast.PCThis;
import abc.aspectj.ast.Pointcut;
import abc.ltl.ast.override.PCArgs_c;
import abc.ltl.ast.override.PCTarget_c;
import abc.ltl.ast.override.PCThis_c;
import abc.meta.ast.MetaNodeFactory_c;

public class LTLNodeFactory_c extends MetaNodeFactory_c implements LTLNodeFactory {

    /**
     * {@inheritDoc}
     */
    public PCFormula PCFormula(Collection<Formal> formals, PCLTLGeneral innerPc, ByteCodePosition pos) {
        return new PCFormula_c(formals, innerPc, pos);
    }

    /**
	 * {@inheritDoc}
	 */
	public PCLTLFinally PCLTLFinally(Position pos, Pointcut pc) {
		return new PCLTLFinally_c(pos,pc);
	}

	/**
	 * {@inheritDoc}
	 */
	public PCLTLGlobally PCLTLGlobally(Position pos, Pointcut pc) {
		return new PCLTLGlobally_c(pos,pc);
	}

    /**
     * {@inheritDoc}
     */
    public PCLTLNext PCLTLNext(Position pos, Pointcut pc) {
        return new PCLTLNext_c(pos,pc);
    }
    
    /**
	 * {@inheritDoc}
	 */
	public PCLTLUntil PCLTLUntil(Position pos, Pointcut pcl, Pointcut pcr) {
		return new PCLTLUntil_c(pos,pcl,pcr);
	}

    /**
     * {@inheritDoc}
     */
    public PCLTLRelease PCLTLRelease(Position pos, Pointcut pcl, Pointcut pcr) {
        return new PCLTLRelease_c(pos,pcl,pcr);
    }
    
    /**
     * {@inheritDoc}
     */
    public ByteCodePosition ByteCodePosition(String className, String sourceFile, int lineNumber) {
        return new ByteCodePosition(className,sourceFile,lineNumber);
    }

//    /**
//     * {@inheritDoc}
//     */
//    @Override
//    public AspectDecl AspectDecl(Position pos, boolean is_privileged, Flags flags, String name, TypeNode superClass, List interfaces, PerClause per, AspectBody body) {
//        return new AspectDecl_c(pos, is_privileged, flags, name, superClass,
//                interfaces, per, body);
//    }

    /**
     * {@inheritDoc}
     */
    public PCLTLAnd PCLTLAnd(Position pos, Pointcut pcl, Pointcut pcr) {
        return new PCLTLAnd_c(pos,pcl,pcr);
    }

    /**
     * {@inheritDoc}
     */
    public PCLTLOr PCLTLOr(Position pos, Pointcut pcl, Pointcut pcr) {
        return new PCLTLOr_c(pos,pcl,pcr);
    }

    /**
     * {@inheritDoc}
     */
    public PCLTLImplies PCLTLImplies(Position pos, Pointcut pcl, Pointcut pcr) {
        return new PCLTLImplies_c(pos,pcl,pcr);
    }

    /**
     * {@inheritDoc}
     */
    public PCLTLEquiv PCLTLEquiv(Position pos, Pointcut pcl, Pointcut pcr) {
        return new PCLTLEquiv_c(pos,pcl,pcr);
    }

    /**
     * {@inheritDoc}
     */
    public PCLTLNot PCLTLNot(Position pos, Pointcut pc) {
        return new PCLTLNot_c(pos,pc);
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public PCThis PCThis(Position pos, ArgPattern pat) {
        PCThis n = new PCThis_c(pos, pat);
        n = (PCThis) n.ext(((AJExtFactory)extFactory()).extPCThis());
        n = (PCThis) n.del(((AJDelFactory)delFactory()).delPCThis());
        return n;
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public PCTarget PCTarget(Position pos, ArgPattern pat) {
        PCTarget n = new PCTarget_c(pos, pat);
        n = (PCTarget) n.ext(((AJExtFactory)extFactory()).extPCTarget());
        n = (PCTarget) n.del(((AJDelFactory)delFactory()).delPCTarget());
        return n;
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public PCArgs PCArgs(Position pos, List pats) {
        PCArgs n = new PCArgs_c(pos, pats);
        n = (PCArgs) n.ext(((AJExtFactory)extFactory()).extPCArgs());
        n = (PCArgs) n.del(((AJDelFactory)delFactory()).delPCArgs());
        return n;
    }
}
