package abc.ltl.ast;

import static abc.ltl.visit.transform.RuntimeRepresentation.PROPOSITION_CONSTANTS_PREFIX;

import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import polyglot.ast.Expr;
import polyglot.ast.Formal;
import polyglot.ast.Node;
import polyglot.ast.Term;
import polyglot.util.CodeWriter;
import polyglot.util.Position;
import polyglot.util.SubtypeSet;
import polyglot.visit.CFGBuilder;
import polyglot.visit.NodeVisitor;
import polyglot.visit.PrettyPrinter;
import rwth.i2.ltlrv.management.ByteCodePosition;
import abc.aspectj.ast.Pointcut;
import abc.aspectj.ast.Pointcut_c;
import abc.ltl.Util;
import abc.ltl.visit.transform.PropositionLabels;

public class PCFormula_c extends Pointcut_c implements PCFormula {

    protected Collection<Formal> formals;
    
    protected PCLTLGeneral innerPc;
    
    protected String originalFormulaAsString; 
    
    protected ByteCodePosition byteCodePosition;
    
    protected Map<Pointcut,Integer> propToId;
    
    public PCFormula_c(Collection<Formal> formals, PCLTLGeneral innerPc, ByteCodePosition byteCodePosition) {
        super(
                new polyglot.util.Position(
                        byteCodePosition.getSourceFile().length()>0 ?
                                byteCodePosition.getSourceFile():
                                byteCodePosition.getClassName(),
                        byteCodePosition.getLine()
                )
        );
        this.formals = formals;
        this.innerPc = innerPc;
        this.originalFormulaAsString = Util.nodeToString(this);
        this.byteCodePosition = byteCodePosition;
        this.propToId = Collections.emptyMap();
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public Node visitChildren(NodeVisitor v) {
        Node n = super.visitChildren(v);
        
        PCLTLGeneral pc = (PCLTLGeneral)visitChild(this.innerPc,v);
        
        return reconstruct(n, pc);
    }
    
    protected Node reconstruct(Node n, PCLTLGeneral innerPc) {
        if(innerPc != this.innerPc) {
            PCFormula_c copy = (PCFormula_c) n.copy();
            copy.innerPc = innerPc;
            return copy;
        } else {
            return n;
        }
    }
    
    /** 
     * Prints the formals declaration.
     * @param w CodeWriter to print to
     * @param pp PrettyPrinter to format with
     */
    public void printFormals(CodeWriter w, PrettyPrinter pp) {
        if(this.formals.size()>0) {
            for (Iterator<Formal> iter = this.formals.iterator(); iter.hasNext();) {
                Formal formal = (Formal) iter.next();
                formal.prettyPrint(w,pp);
                if(iter.hasNext())
                    w.write(", ");
            }
            w.write(": ");
        }
    }
    
    public void prettyPrint(CodeWriter w, PrettyPrinter pp) {
        this.printFormals(w,pp);
        this.innerPc.prettyPrint(w,pp);
    }
    
    public boolean isDynamic() {
        return true;
    }

    public abc.weaving.aspectinfo.Pointcut makeAIPointcut() {
        throw new RuntimeException("operation not supported");
    }

    public Set pcRefs() {
        throw new RuntimeException("operation not supported");
    }

    public Collection<Formal> getFormals() {
        return formals;
    }

    public PCLTLGeneral getInnerPc() {
        return innerPc;
    }

    /**
     * {@inheritDoc}
     */
    public Term entry() {
        throw new RuntimeException("operation not supported");
    }

    /**
     * {@inheritDoc}
     */
    @SuppressWarnings("unused")
    public List acceptCFG(CFGBuilder v, List succs) {
        throw new RuntimeException("operation not supported");
    }

    /**
     * {@inheritDoc}
     */
    public boolean reachable() {
        throw new RuntimeException("operation not supported");
    }

    /**
     * {@inheritDoc}
     */
    @SuppressWarnings("unused")
    public Term reachable(boolean reachability) {
        throw new RuntimeException("operation not supported");
    }

    public ByteCodePosition getByteCodePosition() {
        return byteCodePosition;
    }

    /**
     * {@inheritDoc}
     */
    public Expr formulaConstructorExpr(PropositionLabels vis, Position pos) {
        return innerPc.formulaConstructorExpr(vis,pos, this.formals);
    }

    /**
     * {@inheritDoc}
     */
    public void setPropositionIds(Map<Pointcut, Integer> propToId) {
        this.propToId = propToId;
    }

    /**
     * @param pc
     * @return
     */
    public String propositionLabel(Pointcut pc) {
        assert this.propToId.containsKey(pc);
        return PROPOSITION_CONSTANTS_PREFIX + this.propToId.get(pc);
    }     
    
    /**
     * {@inheritDoc}
     */
    public boolean isPrepareForMultiThreading() {
        return true;
    }

    /**
     * {@inheritDoc}
     */
    public SubtypeSet exceptions() {
        return null;
    }

    /**
     * {@inheritDoc}
     */
    public Term exceptions(SubtypeSet exceptions) {
        return null;
    }

}
