package abc.ltl.ast;

import static abc.ltl.visit.transform.RuntimeRepresentation.FACTORY_LABEL;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import polyglot.ast.Expr;
import polyglot.ast.Formal;
import polyglot.ast.Node;
import polyglot.ast.NodeFactory;
import polyglot.ast.Precedence;
import polyglot.types.SemanticException;
import polyglot.util.CodeWriter;
import polyglot.util.Position;
import polyglot.visit.NodeVisitor;
import polyglot.visit.PrettyPrinter;
import abc.aspectj.ast.Pointcut;
import abc.ltl.visit.transform.PropositionLabels;

public abstract class PCLTLBinary_c extends PCLTLGeneral_c {

	protected Pointcut pcl;
	protected Pointcut pcr;

	public PCLTLBinary_c(Position pos, Pointcut pcl, Pointcut pcr) {
		super(pos);
		this.pcl = pcl;
		this.pcr = pcr;
	}

    @SuppressWarnings("unchecked")
    public Collection mustBind() {
		Collection join = new HashSet();
		join.addAll(pcl.mustBind());
		join.addAll(pcr.mustBind());
		return join;
	}

    @SuppressWarnings("unchecked")
	public Collection mayBind() throws SemanticException {
		Collection join = new HashSet();
		join.addAll(pcl.mayBind());
		join.addAll(pcr.mayBind());
		return join;
	}

	/** Visit the children of the pointcut. */
	public Node visitChildren(NodeVisitor v) {
		 Pointcut pcl = (Pointcut) visitChild(this.pcl, v);
		 Pointcut pcr = (Pointcut) visitChild(this.pcr, v);
		 return reconstruct(pcl,pcr);
	}

	protected abstract Node reconstruct(Pointcut pcl, Pointcut pcr);

	public boolean isDynamic() {
		return true;
	}

    @SuppressWarnings("unchecked")
	public Set pcRefs() {
		Set join = new HashSet();
		join.addAll(pcl.pcRefs());
		join.addAll(pcr.pcRefs());
		return join;
	}

	public Precedence precedence() {
		return Precedence.LITERAL;
	}
    
    /**
     * {@inheritDoc}
     */
    @Override
    public void prettyPrint(CodeWriter w, PrettyPrinter tr) {
        w.write("(");
        print(pcl, w, tr);
        w.write(")" + this.getSymbol() + "(");
        print(pcr, w, tr);
        w.write(")");
    }

    /**
     * @return symbol string for this term constructor 
     */
    protected abstract String getSymbol();

    public Expr formulaConstructorExpr(PropositionLabels vis, Position pos, Collection<Formal> declaredFormals) {
        List<Expr> args = new ArrayList<Expr>(2);
        
        NodeFactory nf = vis.nf();
        
        if(pcl instanceof PCLTLGeneral) {
            //if left argument is a formula, recurse
            args.add(((PCLTLGeneral)pcl).formulaConstructorExpr(vis,pos, declaredFormals));            
        } else {
            //else, if it's a simple pointcut, create expression for that
            Expr left = nf.AmbExpr(pos,vis.propositionLabel(pcl));
            args.add(left);
        }
        if(pcr instanceof PCLTLGeneral) {
            //if right argument is a formula, recurse
            args.add(((PCLTLGeneral)pcr).formulaConstructorExpr(vis,pos, declaredFormals));            
        } else {
            //else, if it's a simple pointcut, create expression for that
            Expr right = nf.AmbExpr(pos,vis.propositionLabel(pcr));
            args.add(right);
        }        
        
        return nf.Call(
                pos,
                nf.AmbReceiver(
                        pos,
                        FACTORY_LABEL
                ),
                this.runtimeClassName(),
                args
        );        
    }

    /**
     * @return
     */
    protected abstract String runtimeClassName();
    
}
