package abc.ltl.ast;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Set;

import polyglot.ast.Expr;
import polyglot.ast.Formal;
import polyglot.ast.Node;
import polyglot.ast.NodeFactory;
import polyglot.ast.Precedence;
import polyglot.types.SemanticException;
import polyglot.util.CodeWriter;
import polyglot.util.Position;
import polyglot.visit.NodeVisitor;
import polyglot.visit.PrettyPrinter;
import abc.aspectj.ast.Pointcut;
import abc.ltl.visit.transform.PropositionLabels;
import static abc.ltl.visit.transform.RuntimeRepresentation.FACTORY_LABEL;

public abstract class PCLTLUnary_c extends PCLTLGeneral_c {

	protected Pointcut pc;

	public PCLTLUnary_c(Position pos, Pointcut pc) {
		super(pos);
		this.pc = pc;
	}

	public Collection mustBind() {
	 	return pc.mustBind();
	}

	public Collection mayBind() throws SemanticException {
		return pc.mayBind();
	}

	/** Visit the children of the pointcut. */
	public Node visitChildren(NodeVisitor v) {
		 Pointcut pc = (Pointcut) visitChild(this.pc, v);
		 return reconstruct(pc);
	}

	protected abstract Node reconstruct(Pointcut pc);

	public boolean isDynamic() {
		return true;
	}

	public Set pcRefs() {
		return pc.pcRefs();
	}

	public Precedence precedence() {
		return Precedence.LITERAL;
	}

    public void prettyPrint(CodeWriter w, PrettyPrinter tr) {
        w.write(this.getSymbol() + "(");
        print(pc, w, tr);
        w.write(")");
    }

    /**
     * @return the symbol for this term constructor
     */
    protected abstract String getSymbol();
    
    public Expr formulaConstructorExpr(PropositionLabels vis, Position pos, Collection<Formal> declaredFormals) {
        List<Expr> args = new ArrayList<Expr>(2);
        
        NodeFactory nf = vis.nf();
        
        if(pc instanceof PCLTLGeneral) {
            //if argument is a formula, recurse
            args.add(((PCLTLGeneral)pc).formulaConstructorExpr(vis,pos, declaredFormals));            
        } else {
            //else, if it's a simple pointcut, create expression for that
            Expr inner = nf.AmbExpr(pos,vis.propositionLabel(pc));
            args.add(inner);
        }
        
        return nf.Call(
                pos,
                nf.AmbReceiver(
                        pos,
                        FACTORY_LABEL
                ),
                this.runtimeClassName(),
                args
        );         
    }

    /**
     * @return
     */
    protected abstract String runtimeClassName();
}
