package abc.ltl.ast.override;

import java.util.ArrayList;
import java.util.List;

import polyglot.ast.AmbTypeNode;
import polyglot.ast.Formal;
import polyglot.ast.Node;
import polyglot.ast.TypeNode;
import polyglot.util.Position;
import abc.aspectj.ast.AmbTypeOrLocal;
import abc.ltl.ast.PointcutUtils;

/**
 * Override to adhere to the StatebindingPointcut interface.
 * @see abc.ltl.ast.override.StatebindingPointcut
 * @author Eric Bodden
 */
public class PCArgs_c extends abc.aspectj.ast.PCArgs_c implements StatebindingPointcut {

    /**
     * @param pos
     * @param pats
     */
    public PCArgs_c(Position pos, List pats) {
        super(pos, pats);
    }
    
    /**
     * {@inheritDoc}
     */
    @SuppressWarnings("unchecked")
    public Node[] patterns() {
        return ((List<Node>)pats).toArray(new Node[0]);        
    }

    /**
     * {@inheritDoc}
     */
    @SuppressWarnings("unchecked")
    public Node replaceBindingsByTypes(Formal... declaredFormals) {
        List newPatterns = new ArrayList();

        for (Object pat : pats) {
            if(!(pat instanceof AmbTypeOrLocal)) {
                throw new RuntimeException("internal error in PCThis_c [1]");
            }
            TypeNode type = ((AmbTypeOrLocal)pat).type();
            
            if(!(type instanceof AmbTypeNode)) {
                throw new RuntimeException("internal error in PCThis_c [2]");
            }
            AmbTypeNode amb = (AmbTypeNode) type;
            String name = amb.name();
            TypeNode boundType = PointcutUtils.bindingNameToType(name,declaredFormals);
            
            if(boundType==null) {
                //"name" has no bound type so "name" maybe *is* a type itself
                newPatterns.add(pat);
            } else {
                newPatterns.add(boundType.copy());
            }
        }
        return reconstruct(newPatterns);
    }
    
}