package abc.ltl.ast.override;

import polyglot.ast.Formal;
import polyglot.ast.Node;


/**
 * Common interface for pointcuts binding state. (this/target/args)
 * 
 * @author Eric Bodden
 */
public interface StatebindingPointcut {

    /**
     * @return The pattern which is used to bind state in this pointcut.
     * Usually of type AmbTypeOrLocal.
     * @see abc.aspectj.ast.AmbTypeOrLocal
     */
    public Node[] patterns();

    /**
     * Returns a copy of this node where all bindings in this/target/args
     * have been replaced by the types of those bindings. E.g. in a pointcut
     * declaration <code>args(i,j)</code> where formals <code>int i, long j</code>
     * have been declared, the result will be <code>args(int,long)</code>.
     * @param declaredFormals list of formals that shall be used to resolve types
     * @return copy of the node where bindings have been replaced by their types   
     */
    public Node replaceBindingsByTypes(Formal... declaredFormals);
    
}