package abc.ltl.formulaextraction;

import rwth.i2.ltlrv.management.ByteCodePosition;
import abc.ltl.AbcExtension;

/**
 * @author Eric Bodden
 *
 * Models a formula.
 */
public class Formula {

    private String formulaString;
    
    private ByteCodePosition positionInBytecode;

    /**
     * @param formulaString String content of formula
     * @param className class name of the enclosing class (dot format)
     * @param sourceFile name of the source file (if known)
     * @param line (approximate) line number
     */
    public Formula(String formulaString, String className, String sourceFile, int line) {
        this(formulaString, new ByteCodePosition(className, sourceFile, line));
    }

    
    /**
     * @param formulaString String content of formula
     * @param positionInBytecode (approximate) position in the bytecode 
     */
    private Formula(String formulaString, ByteCodePosition positionInBytecode) {
        this.formulaString = formulaString;
        this.positionInBytecode = positionInBytecode;
    }

    /**
     * @return representation of formula used in the grammar 
     */
    public String getRepresentationForGrammar() {
        return AbcExtension.FORMULA_TERM_CONSTRUCTOR
            + "("
            + this.positionInBytecode.getRepresentationForGrammar()            
            + formulaString
            + ");";
    }

}
