package abc.ltl.formulaextraction;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.Arrays;

import polyglot.frontend.FileSource;

/**
 * A SyntheticSource is a source defined by some bytearray.
 * 
 * @author Eric Bodden
 */
public class SyntheticSource extends FileSource {

    private byte[] content;
    
    private Reader reader;
    
    /**
     * @param content Content of the source file. 
     * @param filePath imaginary path to the synthetic file according to package
     * 	and class id.
     * @throws IOException cannot occur but has to be there for compatibility with FileSource
     */
    public SyntheticSource(byte[] content, String filePath) throws IOException {
        super(new File(filePath) {
            
            private static final long serialVersionUID = 1L;

            @Override
            public boolean exists() {
                //a synthetic source is simulated to always exist
                return true;
            }
            
        },true);
        
        assert filePath!=null && filePath.length()>0;
        assert content!=null && content.length>0;
        this.content = content;
        reader = null;
    }
    
    /**
     * {@inheritDoc}
     */
    public Reader open() throws IOException {
        if(this.reader==null) {
            this.reader = new InputStreamReader(new ByteArrayInputStream(content));
        }
        return this.reader;
    }
    
    /**
     * {@inheritDoc}
     */
    public void close() throws IOException {
        this.reader.close();
    }
    
    /**
     * {@inheritDoc}
     */
    public boolean equals(Object source) {
        if(!(source instanceof SyntheticSource)) {
            return false;
        }
        return Arrays.equals(this.content,((SyntheticSource)source).content);
    }

    /**
     * {@inheritDoc}
     */
    public int hashCode() {
        return Arrays.hashCode(this.content);
    }
    
    /**
     * {@inheritDoc}
     */
    public String toString() {
        return "<SyntheticSource>";
    }
}
