package abc.ltl.visit.transform;

import static abc.ltl.visit.transform.RuntimeRepresentation.FORMULA_LABEL;
import static abc.ltl.visit.transform.RuntimeRepresentation.REGISTER_FORMULA;
import static abc.ltl.visit.transform.RuntimeRepresentation.RUNTIME_ENVIRONMENT_CLASSNAME;
import static abc.ltl.visit.transform.RuntimeRepresentation.RUNTIME_ENVIRONMENT_SINGLETON_ACCESS;

import java.util.Collections;

import polyglot.ast.Block;
import polyglot.ast.ConstructorDecl;
import polyglot.ast.Node;
import polyglot.ast.NodeFactory;
import polyglot.types.Flags;
import polyglot.visit.NodeVisitor;
import abc.aspectj.ast.AspectDecl;

public class ConstructorDeclaration extends NodeVisitor {

    private NodeFactory nf;

    /**
     * @param nf
     */
    public ConstructorDeclaration(NodeFactory nf) {
        super();
        this.nf = nf;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Node leave(Node old, Node n, NodeVisitor v) {
        if(n instanceof AspectDecl) {
            AspectDecl aspect = (AspectDecl)n;
            
            Block block = nf.Block(n.position());
            
            //constructor block
            
            //need an explicit super-call here
            block = block.append(
                    nf.SuperCall(n.position(),Collections.EMPTY_LIST)
            );
            
            //register formula
            block = block.append(
                    nf.Eval(
                            n.position(),
                            nf.Call(
                                    n.position(),
                                    nf.Call(
                                            n.position(),
                                            nf.AmbReceiver(
                                                    n.position(),
                                                    RUNTIME_ENVIRONMENT_CLASSNAME
                                            ),
                                            RUNTIME_ENVIRONMENT_SINGLETON_ACCESS
                                    ),                                    
                                    REGISTER_FORMULA,
                                    nf.StringLit(
                                            n.position(),
                                            aspect.name()
                                    ),
                                    nf.AmbExpr(
                                            n.position(),
                                            FORMULA_LABEL
                                    )
                            )
                    )
            );

            //constructor decl
            ConstructorDecl constr = nf.ConstructorDecl(
                    n.position(),
                    Flags.PUBLIC,
                    aspect.name(),
                    Collections.EMPTY_LIST,
                    Collections.EMPTY_LIST,
                    block
            );
            
            //add to body
            n = aspect.body(aspect.body().addMember(constr));
       }        
       return super.leave(old, n, v);
    }

}
