package abc.ltl.visit.transform;

import static abc.ltl.visit.transform.RuntimeRepresentation.CURRENT_PROPOSITIONS_LABEL;
import static abc.ltl.visit.transform.RuntimeRepresentation.FACTORY_INSTANTIATION_CALL;
import static abc.ltl.visit.transform.RuntimeRepresentation.FACTORY_INTERFACE_NAME;
import static abc.ltl.visit.transform.RuntimeRepresentation.FACTORY_LABEL;
import static abc.ltl.visit.transform.RuntimeRepresentation.PROPOSITION_SET_CLASS_NAME;
import static abc.ltl.visit.transform.RuntimeRepresentation.RUNTIME_ENVIRONMENT_CLASSNAME;
import static abc.ltl.visit.transform.RuntimeRepresentation.RUNTIME_ENVIRONMENT_SINGLETON_ACCESS;

import java.util.Collections;

import polyglot.ast.ClassBody;
import polyglot.ast.Expr;
import polyglot.ast.Node;
import polyglot.ast.NodeFactory;
import polyglot.ast.TypeNode;
import polyglot.types.Flags;
import polyglot.visit.NodeVisitor;
import abc.aspectj.ast.AspectBody;

public class GeneralFieldDecls extends NodeVisitor {

    private NodeFactory nf;
    /**
     * @param nf
     */
    public GeneralFieldDecls(NodeFactory nf) {
        super();
        this.nf = nf;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Node leave(Node old, Node n, NodeVisitor v) {
        if(n instanceof AspectBody) {
            ClassBody body = (AspectBody)n;

            TypeNode propSetType = nf.AmbTypeNode(n.position(),PROPOSITION_SET_CLASS_NAME);
            
            //initializer
            Expr propSetInit = nf.New(n.position(), propSetType, Collections.EMPTY_LIST);
            
            //add field declaration            
            body = body.addMember(
                    nf.FieldDecl(n.position(), Flags.PRIVATE, propSetType, CURRENT_PROPOSITIONS_LABEL, propSetInit)
            );

            TypeNode factoryType = nf.AmbTypeNode(n.position(),FACTORY_INTERFACE_NAME);
            
            Expr factoryInit = nf.Call(
                    n.position(),
                    nf.Call(
                            n.position(),
                            nf.AmbReceiver(
                                    n.position(),
                                    RUNTIME_ENVIRONMENT_CLASSNAME
                            ),
                            RUNTIME_ENVIRONMENT_SINGLETON_ACCESS
                    ),
                    FACTORY_INSTANTIATION_CALL
            ); 
            
            body = body.addMember(
                    nf.FieldDecl(n.position(), Flags.PRIVATE.set(Flags.FINAL.set(Flags.STATIC)), factoryType, FACTORY_LABEL, factoryInit)
            );
            
            n = body;
            
        }
        return super.leave(old, n, v);
    }
}
