package abc.ltl.visit.transform;

import java.util.HashSet;
import java.util.Set;

import polyglot.ast.Node;
import polyglot.types.TypeSystem;
import polyglot.util.Position;
import polyglot.visit.NodeVisitor;
import abc.aspectj.ast.AJNodeFactory;
import abc.aspectj.ast.AspectDecl;
import abc.aspectj.ast.PCBinary;
import abc.aspectj.ast.Pointcut;
import abc.ltl.ast.PCFormula;
import abc.ltl.ast.PCLTLGeneral;

public abstract class PointcutsVisitor extends NodeVisitor {

    protected AJNodeFactory nf;

    protected TypeSystem ts;

    protected Set<Pointcut> pointcuts;

    protected PCFormula formula;
    
    protected AspectDecl enclosingAspect;
    
    private boolean inFormula = false;

    /**
     * @param nf
     */
    public PointcutsVisitor(AJNodeFactory nf, TypeSystem ts) {
        this.nf = nf;
        this.ts = ts;
        this.pointcuts = new HashSet<Pointcut>();
    }
    
    /**
     * {@inheritDoc}
     */
    public NodeVisitor enter(Node n) {
        if (n instanceof AspectDecl) {
            // store reference to aspect
            this.enclosingAspect = (AspectDecl) n;
        } else if (n instanceof PCFormula) {
            // store reference to formula
            this.formula = (PCFormula) n;
            this.inFormula = true;
        } else if (n instanceof Pointcut && !(n instanceof PCLTLGeneral)) {
            // if we have a simply pointcut, store it
            if(this.inFormula) {
                //... if we are inside the formula
                this.pointcuts.add((Pointcut) n);
            }
        }
        return super.enter(n);
    }

    /**
     * {@inheritDoc}
     */
    public Node visitEdge(Node parent, Node child) {
        // if the parent pointcut is already in the list, don't recurse any
        // deeper
        if (this.pointcuts.contains(parent)) {
            return child;
        }
        return super.visitEdge(parent, child);
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public Node leave(Node old, Node n, NodeVisitor v) {
        if (n instanceof PCFormula) {
            //we left the formula
            this.inFormula = false;
        } 
        return super.leave(old, n, v);
    }
    
}
