package abc.ltl.visit.transform;

import static abc.ltl.visit.transform.RuntimeRepresentation.FORMULA_INTERFACE_NAME;
import static abc.ltl.visit.transform.RuntimeRepresentation.FORMULA_LABEL;
import static abc.ltl.visit.transform.RuntimeRepresentation.PROPOSITION_CONSTANTS_PREFIX;
import static abc.ltl.visit.transform.RuntimeRepresentation.PROPOSITION_INTERFACE_TYPE;

import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import polyglot.ast.ClassDecl;
import polyglot.ast.Expr;
import polyglot.ast.FieldDecl;
import polyglot.ast.Node;
import polyglot.types.Flags;
import polyglot.types.TypeSystem;
import polyglot.util.Position;
import polyglot.visit.NodeVisitor;
import abc.aspectj.ast.AJNodeFactory;
import abc.aspectj.ast.AspectDecl;
import abc.aspectj.ast.Pointcut;
import abc.ltl.ast.PointcutUtils;

public class PropositionLabels extends PointcutsVisitor {

    private int lastId;

    private Map<Pointcut,Integer> propositionToId;

    /**
     * @param nf
     */
    public PropositionLabels(AJNodeFactory nf, TypeSystem ts) {
        super(nf, ts);
        this.lastId = 0;
        this.propositionToId = new HashMap<Pointcut,Integer>();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Node leave(Node old, Node n, NodeVisitor v) {
        if(n instanceof AspectDecl) {
            for (Pointcut pc : pointcuts) {
                propositionToId.put(pc,lastId);
                lastId++;
            }
            
            ClassDecl aspect = (AspectDecl)n;
            
            final Position pos = n.position();
            
            //generate proposition constants
            for(Entry<Pointcut,Integer> entry: propositionToId.entrySet()) {
                //Expr init = nf.StringLit(pos, Util.nodeToString(entry.getKey()));
                Pointcut pc = entry.getKey();
                
                Expr init = PointcutUtils.propositionInstantiationExprForPointcut(
                        pc,
                        this,
                        formula.getFormals(),
                        pos
                );
                
                FieldDecl field = nf.FieldDecl(
                        pos,
                        Flags.PRIVATE.set(Flags.FINAL),
                        nf.AmbTypeNode(
                                pos,
                                PROPOSITION_INTERFACE_TYPE
                        ),
                        propositionLabel(pc),
                        init
                );
                        
                aspect = aspect.body(aspect.body().addMember(field));                
            }
            
            //generate "original formula" constant
            FieldDecl field = nf.FieldDecl(
                    pos,
                    Flags.PRIVATE.set(Flags.FINAL),
                    nf.AmbTypeNode(pos,FORMULA_INTERFACE_NAME),
                    FORMULA_LABEL,
                    formula.formulaConstructorExpr(this, pos)
            );
            aspect = aspect.body(aspect.body().addMember(field));
            
            n = aspect;
        }            
        return super.leave(old, n, v);
    }

    /**
     * @param pc
     * @return
     */
    public String propositionLabel(Pointcut pc) {
        return PROPOSITION_CONSTANTS_PREFIX + idOfProposition(pc);
    }    
    
    private int idOfProposition(Pointcut proposition) {
        assert propositionToId.containsKey(proposition);        
        return propositionToId.get(proposition);
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public void finish() {
        //store copy of map in the formula
        formula.setPropositionIds(propositionToId);
    }
    
    public AJNodeFactory nf() {
        return nf; 
    }
    
    public TypeSystem ts() {
        return ts;
    }
    
}
