package abc.ltl.visit.transform;


public interface RuntimeRepresentation {

    /** GLOBAL RUNTIME SPECS */
    
    /** class name of the runtime class, which manages the factory and formulae */
    public static final String RUNTIME_ENVIRONMENT_CLASSNAME = "VerificationRuntime";
    /** method name for accessing the singleton instance of the runtime environment */
    public static final String RUNTIME_ENVIRONMENT_SINGLETON_ACCESS = "getInstance";
    
    /* FACTORY MANAGEMENT */
    
    /** declared type of factory */
    public static final String FACTORY_INTERFACE_NAME = "IFormulaFactory";
    /** method to retrieve the factory instance */
    public static final String FACTORY_INSTANTIATION_CALL = "getFactory";
    /** label for the aspect-internal reference to the factory */
    public static final String FACTORY_LABEL = "factory";
    
    /* FORMULA INSTANTIATION */
    
    /** name of method which converts into negation normal form */
    public static final String NNF_METHOD_NAME = "negationNormalForm";
    /** interface name for states of the alternating finite automaton */
    public static final String AFA_STATE_INTERFACE_NAME = "IAFATerm";
    /** interface name for formulae */
    public static final String FORMULA_INTERFACE_NAME = "IFormula";
    /** proposition class */
    public static final String PROPOSITION_CLASS_NAME = "Proposition";
    /** <i>and</i> operator class */
    public static final String OP_AND_CLASS_NAME = "And";
    /** <i>or</i> operator class */
    public static final String OP_OR_CLASS_NAME = "Or";
    /** <i>not</i> operator class */
    public static final String OP_NOT_CLASS_NAME = "Not";
    /** <i>implies</i> operator class */
    public static final String OP_IMPLIES_CLASS_NAME = "Impl";
    /** <i>equivalent</i> operator class */
    public static final String OP_EQUIV_CLASS_NAME = "Equivalent";
    /** <i>finally</i> operator class */
    public static final String OP_F_CLASS_NAME = "F";
    /** <i>globally</i> operator class */
    public static final String OP_G_CLASS_NAME = "G";
    /** <i>next</i> operator class */
    public static final String OP_X_CLASS_NAME = "X";
    /** <i>until</i> operator class */
    public static final String OP_U_CLASS_NAME = "Until";
    /** <i>release</i> operator class */
    public static final String OP_R_CLASS_NAME = "Release";
    
    /* FORMULA MANAGEMENT */
    
    /** label for the constant formula */
    public static final String FORMULA_LABEL = "formula";

    /** call to register a formula */
    public static final String REGISTER_FORMULA = "registerFormula";
    
    /** call to update a formula */
    public static final String UPDATE_FORMULA = "updateFormula";

    /* PROPOSITION MANAGEMENT */
    
    /** prefix for labels of propositions */
    public static final String PROPOSITION_CONSTANTS_PREFIX = "prop";
    
    /** name of the class of the if-closure to use */
    public static final String PROPOSITION_IF_CLOSURE_CLASS_NAME = "rwth.i2.ltlrv.afastate.interfaze.IIfClosure";
    
    /** type holding the type UserCausedException */
    public static final String USER_CAUSED_EXCEPTION_QUALIFIER_NAME = "IIfClosure";

    /** type name of UserCausedException */
    public static final String USER_CAUSED_EXCEPTION_CLASS_NAME = "UserCausedException";

    /** name of the evaluation method to override in if-closure */
    public static final String PROPOSITION_IF_CLOSURE_EVAL_METHOD_NAME = "satisfiedUnderBindings";
    
    /** name of the variable names to override in if-closure */
    public static final String PROPOSITION_IF_CLOSURE_VARIABLES_METHOD_NAME = "variableNames";

    /** variable name used for after returning/throwing if a type name is given */
    public static final String DUMMY_VARIABLE_NAME = "_";

    /** label of set of current propositions */ 
    public static final String CURRENT_PROPOSITIONS_LABEL = "currentProps";
    
    /** (declared and actual) type of set of current propositions */
    public static final String PROPOSITION_SET_CLASS_NAME = "PropositionSet";
    
    /** method to add a proposition to the set */
    public static final String CURRENT_PROPOSITIONS_ADD_METHOD = "add";

    /** method to clear the proposition set */
    public static final String CURRENT_PROPOSITIONS_CLEAR_METHOD = "clear";
    
    /* BINDING MANAGEMENT */
    
    /** label for local variable holding the bindings */ 
    public static final String BINDINGS_MAP_LOCAL_VARIABLE_NAME = "bindings";

    /** declared type of bindings map */
    public static final String BINDINGS_MAP_DECLARED_TYPE_NAME = "WeakValuesMap";
    
    /** actual type of bindings map */
    public static final String BINDINGS_MAP_ACTUAL_TYPE_NAME = "WeakValuesHashMap";

    /** method to map a binding */
    public static final String BINDINGS_MAP_LOCAL_VARIABLE_PUT_METHOD = "put";
    
    /** method to retrieve a binding */
    public static final String BINDINGS_MAP_LOCAL_VARIABLE_GET_METHOD = "get";

    //TODO comment
    public static final String PROPOSITION_INTERFACE_TYPE = "IProposition";    
    
    public static final String PROPOSITION_SPECIALIZE_METHOD = "specializeBindings";
    
    /* TRANSITION MANAGEMENT */
    
    /** method to perform transition */
    public final static String TRANSITION_METHOD = "alternatingTransition";
    
    /* GENERIC IMPORTS */
    
    /** import statements */
    public static final String[] PACKAGE_IMPORTS = {
        "rwth.i2.ltlrv.data",
        "rwth.i2.ltlrv.management",
        "rwth.i2.ltlrv.afastate.interfaze",
        "rwth.i2.ltlrv.formula.interfaze"
    };

}
